/*
Module : DDXGETFOLDER.CPP
Purpose: implementation for a MFC DDX_ routine to get a directory/folder
         Using the file open/save as common dialogs or the newer Shell API
         SHBrowseForFolder

Created: PJN / DDXGETFOLDER/1 / 26-03-1997
History: None

Copyright (c) 1997 by PJ Naughter.  
All rights reserved.

*/

/////////////////////////////////  Includes  //////////////////////////////////
#include "stdafx.h"
#include "resource.h"
#include "ddxgetfolder.h"
#include "dlgdir.h"



///////////////////////////////// defines /////////////////////////////////////

#define GETFOLDER_EDIT_ID 100


BEGIN_MESSAGE_MAP(CModifyButton, CButton)
  //{{AFX_MSG_MAP(CModifyButton)
  ON_CONTROL_REFLECT(BN_CLICKED, OnClicked)
  //}}AFX_MSG_MAP
END_MESSAGE_MAP()

        
CModifyButton::CModifyButton()
{
  m_pBuddy = NULL;
  m_bFirstCall = TRUE;
}


void CModifyButton::SetBuddy(CGetFolderControl* pBuddy)
{
  m_pBuddy = pBuddy;
}


BOOL CModifyButton::PreTranslateMessage(MSG* pMsg) 
{                     
  //create the tooltip
  if (m_bFirstCall)
  {
    m_ToolTip.Create(this);
    m_ToolTip.Activate(TRUE);
    m_ToolTip.AddTool(this, IDS_DDX_GFLDR_TT_MODIFY);
    m_bFirstCall = FALSE;
  }
  
  //give the tooltip a chance to handle the message
  m_ToolTip.RelayEvent(pMsg);

  return CButton::PreTranslateMessage(pMsg);
}


void CModifyButton::OnClicked() 
{
  if (m_pBuddy)
    m_pBuddy->Edit();
  else
    TRACE0("CModifyButton: No auto buddy defined\n");
}                        






BEGIN_MESSAGE_MAP(CGetFolderControl, CStatic)
  //{{AFX_MSG_MAP(CGetFolderControl)
  //}}AFX_MSG_MAP
END_MESSAGE_MAP()


CGetFolderControl::CGetFolderControl()
{
}


BOOL CGetFolderControl::SubclassEdit(HWND hEdit)
{   
  //test our inputs
  ASSERT(this);
  if (!IsWindow(hEdit))
  {
    ASSERT(FALSE);
    TRACE0("CGetFolderControl::SubclassEdit -- window handle is invalid!\n");
    return FALSE;
  }                
  
  //subclass the control
  if (SubclassWindow(hEdit))
  {
    //This control can only be used with a read only edit control
    LONG lStyle = GetWindowLong(GetSafeHwnd(), GWL_STYLE);
    if (!(lStyle & ES_READONLY))
    {
      TRACE0("CGetFolderControl::SubclassEdit -- ES_READONLY style should be set for the edit control\n");
      ASSERT(FALSE);
      return FALSE;
    }

    return AddEditButton();
  }
  else
  {
    TRACE0("CGetFolderControl::SubclassEdit -- Could not subclass edit control!\n");
    ASSERT(FALSE);
    return FALSE;
  }
}


BOOL CGetFolderControl::AddEditButton()
{
  CRect Rect;
  GetWindowRect(Rect);
  GetParent()->ScreenToClient(Rect);
  Rect.left = Rect.right;
  Rect.right = Rect.left + (Rect.Height()*8/10);  //width is 8/10 of height

  //dynamically create the edit button control
  CString sEditControlText;
  if (!sEditControlText.LoadString(IDS_DDX_GFLDR_EDIT_TEXT))
    ASSERT(FALSE);
  BOOL bSuccess = m_Edit.Create(sEditControlText, WS_VISIBLE | WS_CHILD | WS_GROUP, Rect, GetParent(), GETFOLDER_EDIT_ID);

  //tell the button to call this class when it is clicked
  m_Edit.SetBuddy(this);

  //ensure it is using the same font as the parent
  m_Edit.SetFont(GetParent()->GetFont());

  return bSuccess;
}


void CGetFolderControl::Edit()
{
  //bring up the common file open dialog to select the folder

  BYTE WinMajorVersion = LOBYTE(LOWORD(GetVersion()));
  if ((WinMajorVersion >= 4) && ((m_dwFlags & GFLDR_OLD_STYLE_DIALOG) == 0)) //Running on Windows 95 shell and new style requested
  {
    TCHAR sDisplayName[_MAX_PATH];
    BROWSEINFO bi;
    bi.hwndOwner = m_hWnd;
    bi.pidlRoot = NULL;
    bi.lpszTitle =  m_sTitle;
    bi.pszDisplayName = sDisplayName;
    bi.ulFlags = BIF_RETURNONLYFSDIRS;
    bi.lpfn = NULL;
    bi.lParam = 0;
	  LPITEMIDLIST pItemIDList = SHBrowseForFolder(&bi);

    if (pItemIDList)
    {
      TCHAR sPath[_MAX_PATH];
      if (SHGetPathFromIDList(pItemIDList, sPath))
        SetWindowText(sPath);

      //avoid memory leaks by deleting the PIDL
      //using the shells task allocator
      IMalloc* pMalloc;
      if (SHGetMalloc(&pMalloc) != NOERROR)
      {
        TRACE("Failed to get pointer to shells task allocator");
        return;
      }
      pMalloc->Free(pItemIDList);
      if (pMalloc)
        pMalloc->Release();
    }
  }
  else  //Use old style if requested or when running on NT 3.51 where we have no choice
  {
    CString sInitialDir;
    GetWindowText(sInitialDir);

    CFileDirDialog dlg(FALSE, NULL, NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT | OFN_ENABLETEMPLATE, NULL, this);
    dlg.m_ofn.Flags &= ~OFN_EXPLORER; //Turn of the explorer style customisation
    dlg.m_ofn.hInstance = AfxGetInstanceHandle();
    dlg.m_ofn.lpTemplateName = MAKEINTRESOURCE(FILEOPENORD);
    dlg.m_ofn.lpstrInitialDir = sInitialDir;
    dlg.m_ofn.lpstrTitle = m_sTitle;
    if (dlg.DoModal() == IDOK)
    {
      dlg.m_ofn.lpstrFile[dlg.m_ofn.nFileOffset-1]=0; //Nuke the "Junk" text filename
      SetWindowText(dlg.m_ofn.lpstrFile);
    }
  }
}


void CGetFolderControl::SetFlags(DWORD dwFlags, const CString& sTitle) 
{ 
  m_dwFlags = dwFlags; 
  m_sTitle = sTitle;
}



void DDX_GetFolderControl(CDataExchange* pDX, int nIDC, CGetFolderControl& rCGetFolderControl, DWORD dwFlags, const CString& sTitle)
{
  HWND hWndCtrl = pDX->PrepareEditCtrl(nIDC);
  if (!pDX->m_bSaveAndValidate && rCGetFolderControl.m_hWnd == NULL)    // not subclassed yet
  {
    if (!rCGetFolderControl.SubclassEdit(hWndCtrl))
    {
      ASSERT(FALSE);      // possibly trying to subclass twice ?
      AfxThrowNotSupportedException();
    }
  }
  rCGetFolderControl.SetFlags(dwFlags, sTitle);
}
