
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/* fixbug.h								  */
/*									  */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/*									  */
/* Win-Fix-Bug V1.03							  */
/* Copyright (c) Third Eye Software 1996, 1997				  */
/* All Rights Reserved.							  */
/*									  */
/* Email: eye3@escape.com						  */
/* WWW:   http://www.escape.com/~eye3   				  */
/*									  */
/* C/C++ header file for the Win-Fix-Bug programming library		  */
/*									  */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

#ifndef _fixbug_h
#define _fixbug_h

#ifdef __cplusplus
extern "C" {
#endif

/* break point handler function */
typedef DWORD (*BPHANDLER)(HANDLE hProcess, CONTEXT *pContext, PVOID pvUser);

/* break point handler return values */
#define BPH_CLEAR	 0x001
#define BPH_REENABLE	 0x002
#define BPH_DISABLE	 0x003	              /* not valid from a SS handler */

BOOL FixBugInit(void);                        /* initialize the library     */
DWORD FBGetLastError(void);		      /* get error code             */
LPCSTR FBErrorStr(DWORD dwCode);	      /* get string for error       */

BOOL FixBugRun(LPTSTR  lpCommandLine,	      /* run a process under fixbug */
	       LPCTSTR lpCurrentDirectory,
	       DWORD   dwFlags);
/* flag values */
#define FBR_DEBUG_SUBPROCESSES 0x0001
#define FBR_REUSE_CONSOLE      0x0002

DWORD BreakPointSet(BPHANDLER bpHandler,      /* set a break point          */
		    LPCTSTR   lpExeName,
		    LPCTSTR   lpDllName,
		    LPCSTR    lpProcName,
		    DWORD     dwOffset,
		    DWORD     dwExpectedByte,
		    PVOID     pvUser);
/* expected byte */
#define EB_ANY 0x80000000

BOOL BreakPointClear(DWORD dwBreakpointId);   /* clear a break point        */
BOOL BreakPointDisable(DWORD dwBreakpointId); /* disable a break point      */
BOOL BreakPointEnable(DWORD dwBreakpointId);  /* enable a break point       */

BOOL SingleStep(BPHANDLER bpHandler, PVOID pvUser); /* single step a thread */

BOOL PatchBytes(LPCTSTR       lpExeName,      /* patch bytes of a process   */
		LPCTSTR       lpDllName,
		LPCSTR	      lpProcName,
		DWORD	      dwOffset,
		const LPBYTE  lpbNew,
		DWORD	      dwLen,
		const LPBYTE  lpbExpected);

DWORD GetModuleBase(LPCTSTR lpExeName,        /* get base address of module */
		    LPCTSTR lpDllName);

BOOL  ReadFunctionArgs(HANDLE hProcess,	      /* read function arguments    */
		       DWORD dwAddress,
		       LPCSTR lpFormat,
		       ...);

DWORD ReadStringA(HANDLE hProcess,            /* read an ascii string       */
		  DWORD dwAddress,
		  LPSTR lpStr,
		  LPDWORD pcchStr);
DWORD ReadStringW(HANDLE hProcess,            /* read a unicode string      */
		  DWORD dwAddress,
		  LPWSTR lpStr,
		  LPDWORD pcchStr);

  
/* error codes */
#define FB_ERROR_SUCCESS	       0	  /* 0 means no error       */
#define FB_ERROR_EVALUATION	       0x80000000 /* eval library expired   */
#define FB_ERROR_INTERNAL	       0x80000001 /* internal error	    */
#define FB_ERROR_UNINIT	               0x80000002 /* lib not initialized    */
#define FB_ERROR_ALREADY_INIT	       0x80000003 /* lib already initializd */
#define FB_ERROR_OSVERSION	       0x80000004 /* unsupported os version */
#define FB_ERROR_INVALID_PARAMETER     0x80000005 /* bad argument to api    */
#define FB_ERROR_CREATE_PROCESS        0x80000006 /* create process failed  */
#define FB_ERROR_DUPLICATE_BREAKPOINT  0x80000007 /* duplicate break pt	    */
#define FB_ERROR_RUNNING	       0x80000008 /* already in FixBugRun() */
#define FB_ERROR_NOT_IN_HANDLER        0x80000009 /* SingleStep in handler  */
#define FB_ERROR_ALREADY_SINGLE_STEP   0x8000000a /* already SS the thread  */
#define FB_ERROR_BUFSIZE	       0x8000000b /* buffer too small	    */

#ifdef __cplusplus
}
#endif

#endif /* _fixbug_h */
