
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * wfb.c - Win-Fix-Bug sample program
 *
 * Demonstrates how to set a break point and single step through code
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * (c) Copyright Third Eye Software 1996-7.  All rights reserved.
 * http://www.escape.com/~eye3
 *
 */

#define STRICT
#define WIN32_LEAN_AND_MEAN
#include <stdio.h>
#include <windows.h>
#include <fixbug.h>

/* print a register dump for the stepped thread */
void print_regs(CONTEXT *pc)
{
  printf("Register Dump:\n");
  printf("eax=%.8x ebx=%.8x ecx=%.8x edx=%.8x esi=%.8x edi=%.8x\n",
	 pc->Eax, pc->Ebx, pc->Ecx, pc->Edx, pc->Esi, pc->Edi);
  printf("ebp=%.8x esp=%.8x eip=%.8x\n",
	 pc->Ebp, pc->Esp, pc->Eip);
}

/* called for each instruction single stepped */
DWORD sshandler(HANDLE hp, CONTEXT *pc, void *p)
{
  static int x = 0;

  print_regs(pc);   
  
  /* step over 8 instructions */
  if (++x < 8)
    return BPH_REENABLE;
  else
    return BPH_CLEAR;
}

DWORD bphandler(HANDLE hp, CONTEXT *pc, void *p)
{
  print_regs(pc);   
  SingleStep(sshandler, p);
  return BPH_CLEAR;
}

int main(int argc, char *argv[])
{
  char *pszCommand = "main.exe";
  
  if (!FixBugInit()) {
    printf("Could not initialize Win-Fix-Bug (%s)\n",
	   FBErrorStr(FBGetLastError()));
    return 1;
  }

  BreakPointSet(bphandler, pszCommand, NULL, "do_work", 0, EB_ANY, NULL);

  FixBugRun(pszCommand, NULL, 0);

  printf("hit <enter> to exit\n");
  getchar();

  return 0;
}

