
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * wfb.c - Win-Fix-Bug sample program
 *
 * Demonstrates how to set break points to spy on all calls to a dll.
 * This is a general technique which can be used on any dll, however
 * due to a limitation in Windows 95, break points can not be set to
 * addresses above 0x80000000.  Unfortunately many system DLLs are
 * located at these addresses such as KERNEL32.DLL, USER32.DLL, and
 * GDI32.DLL. However, you may set break points in these DLLs under
 * Windows NT.
 *
 * Use the W32INFO program to check a DLLs preferred base address
 * to see if you can set break points in it under Windows 95.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * (c) Copyright Third Eye Software 1996-7.  All rights reserved.
 * http://www.escape.com/~eye3
 *
 */

#define STRICT
#define WIN32_LEAN_AND_MEAN
#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <fixbug.h>


DWORD handle_ret(HANDLE hp, CONTEXT *pc, void *p)
{
  printf(" = %d\n", pc->Eax);
  return BPH_CLEAR;
}

DWORD handle_call(HANDLE hp, CONTEXT *pc, void *p)
{
  DWORD n;
  int j;
  DWORD retaddr;
  
  /* the first argument is at Esp+4 */
  /*  ReadProcessMemory(hp, (PVOID)(pc->Esp+4), &j, 4, &n); */
  ReadFunctionArgs(hp, pc->Esp+4, "i", &j);
  printf("call to do_work%d(%d)", (int)p, j);
  
  /* set a break point at the return address */
  /* the return address is at Esp */
  ReadProcessMemory(hp, (PVOID)pc->Esp, &retaddr, 4, &n);
  BreakPointSet(handle_ret, NULL, NULL, "", retaddr, EB_ANY, NULL);
  
  return BPH_REENABLE;
}

DWORD handle_args(HANDLE hp, CONTEXT *pc, void *parm)
{
  unsigned char a;
  unsigned short b;
  unsigned int c;
  unsigned __int64 d;
  float e;
  double f;
  long double g;

  DWORD p, q;

  CHAR *pbufA;
  WCHAR *pbufW;
  
  DWORD cb;
  
  ReadFunctionArgs(hp, pc->Esp+4, "csilfddii", &a, &b, &c, &d, &e, &f, &g,
		   &p, &q);
  
  printf("call to all_args(%#x, %#x, %#x, %#x%.8x, %e, %e, %e, %#x, %#x)\n",
	 a, b, c, (int)(d>>32), (int)d, e, f, g, p, q);

  /* read an ascii string */
  cb = 0;
  ReadStringA(hp, p, NULL, &cb);
  if (FBGetLastError() == FB_ERROR_BUFSIZE) {
    pbufA = (CHAR *)malloc(cb);
    ReadStringA(hp, p, pbufA, &cb);
    printf("read ascii string - \"%s\"\n", pbufA);
    free(pbufA);
  }

  /* read a unicode string */
  cb = 0;
  ReadStringW(hp, q, NULL, &cb);
  if (FBGetLastError() == FB_ERROR_BUFSIZE) {
    pbufW = (WCHAR *)malloc(cb * sizeof(WCHAR));
    ReadStringW(hp, q, pbufW, &cb);
    printf("read unicode string - \"%ws\"\n", pbufW);
    free(pbufW);
  }
  
  return BPH_REENABLE;
}

int main(int argc, char *argv[])
{
  char *pszCmd = "main.exe";
  char *pszLib = "dll.dll";
  
  printf("spying on %s calls to dll %s\n", pszCmd, pszLib);

  if (!FixBugInit()) {
    printf("Could not initialize Win-Fix-Bug (%s)\n",
	   FBErrorStr(FBGetLastError()));
    return 1;
  }
  
  BreakPointSet(handle_call, pszCmd, pszLib, "do_work0", 0, EB_ANY, (PVOID)0);
  BreakPointSet(handle_call, pszCmd, pszLib, "do_work1", 0, EB_ANY, (PVOID)1);
  BreakPointSet(handle_call, pszCmd, pszLib, "do_work2", 0, EB_ANY, (PVOID)2);

  BreakPointSet(handle_args, pszCmd, pszLib, "all_args", 0, EB_ANY, 0);

  FixBugRun(pszCmd, NULL, 0);
  
  printf("hit <enter> to exit\n");
  getchar();
  
  return 0;
}

