
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * wfb.c - Win-Fix-Bug sample program
 *
 * Demonstrates how to set break points and use FixBugRun() from
 * a GUI application
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * (c) Copyright Third Eye Software 1996-7.  All rights reserved.
 * http://www.escape.com/~eye3/
 *
 */

#define STRICT
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <process.h>
#include <fixbug.h>
#include "wfb.h"


static char szAppname[]="WFB";
static HWND hWindow;
static HWND hListWnd;
static HINSTANCE hInstance;

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* Win-Fix-Bug specific code */

static char *pszCmd = "notepad.exe";
static char *pszDll = "comdlg32.dll";
static char *pszExp[] = {
  "ChooseColorA",
  "ChooseColorW",
  "ChooseFontA",
  "ChooseFontW",
  "CommDlgExtendedError",
  "FindTextA",
  "FindTextW",
  "GetFileTitleA",
  "GetFileTitleW",
  "GetOpenFileNameA",
  "GetOpenFileNameW",
  "GetSaveFileNameA",
  "GetSaveFileNameW",
  "PageSetupDlgA",
  "PageSetupDlgW",
  "ReplaceTextA",
  "ReplaceTextW",
  "WEP"
};

static int bRunning = 0;

DWORD handle_api(HANDLE hProcess, CONTEXT *pContext, PVOID nAPI)
{
  char buf[128];
  wsprintf(buf, "%s()", pszExp[(int)nAPI]);
  SendMessage(hListWnd, LB_ADDSTRING, 0, (LPARAM)buf);
  return BPH_REENABLE;
}


void SetBreakPoints(void)
{
  int i, n;

  n = sizeof(pszExp) / sizeof(pszExp[0]);

  /* set a break point at each entry point in the dll */
  for (i=0; i < n; i++) {
    BreakPointSet(handle_api, pszCmd, pszDll, pszExp[i], 0, EB_ANY, (PVOID)i);
  }
}

/* Run the application under Win-Fix-Bug.
 * Since FixBugRun() does not return until the app has exited
 * it should be called in a thread other than the one processing
 * the GetMessage() loop.
 */
void Run(void *p)
{
  char buf[128];
  wsprintf(buf, "Running %s spying on calls to dll %s", pszCmd, pszDll);
  SendMessage(hListWnd, LB_ADDSTRING, 0, (LPARAM)buf);
  FixBugRun(pszCmd, NULL, 0);
  SendMessage(hListWnd, LB_ADDSTRING, 0, (LPARAM)"Application exited");
  bRunning = 0;
  EnableMenuItem(GetMenu(hWindow), IDM_RUN, MF_BYCOMMAND|MF_ENABLED);
}

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
  switch (message) {
    case WM_SIZE:
      MoveWindow(hListWnd,2,2,LOWORD(lParam)-4,HIWORD(lParam)-4,TRUE);
      return 0;
      
    case WM_CREATE:
      return 0;

    case WM_CLOSE: {
      int bExit;
      
      if (bRunning) {
	bExit = (MessageBox(hwnd,
			    "Application is still running under us.\n"
			    "\nDo you wish to terminate the application\n"
			    "and loose any unsaved information?\n",
			    szAppname, MB_YESNO) == IDYES);
      } else {
	bExit = TRUE;
      }
      
      if (bExit)
	DestroyWindow(hwnd);
      return 0;
    }
    
    case WM_DESTROY:
      PostQuitMessage(0);
      return 0;
      
    case WM_COMMAND:
      switch (wParam) {
	case IDM_RUN: 
	  if (!bRunning) {
	    EnableMenuItem(GetMenu(hwnd), IDM_RUN, MF_BYCOMMAND|MF_GRAYED);
	    bRunning = 1;
	    _beginthread(Run, 16768, 0);
	  }
	  return 0;
	case IDM_EXIT:
	  PostMessage(hwnd, WM_CLOSE, 0, 0);
	  return 0;
	case IDM_ABOUT:
	  MessageBox(hwnd,
		     "Win-Fix-Bug Test Application\n"
		     "\n"
		     " Copyright 1996 Third Eye Software\n",
		     szAppname, MB_OK | MB_ICONINFORMATION);
	  return 0;
	  break;
      }
  }
  
  return DefWindowProc(hwnd, message, wParam, lParam);
}

BOOL InitApplication(HINSTANCE hInstance)
{
  WNDCLASS wc;
  
  wc.style = 0;
  wc.lpfnWndProc = WndProc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hInstance = hInstance;
  wc.hIcon = NULL;
  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)COLOR_APPWORKSPACE;
  wc.lpszMenuName =  szAppname;
  wc.lpszClassName = szAppname;
  
  return RegisterClass(&wc);
}

BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
  if (!FixBugInit()) {
    char buf[128];
    wsprintf(buf, "Couldn't initialize Win-Fix-Bug (%s)",
	     FBErrorStr(FBGetLastError()));	     
    MessageBox(NULL, buf, NULL, MB_OK);
    return 0;
  }
  
  hWindow = CreateWindow(szAppname,
			 "Win-Fix-Bug Test App",
			 WS_OVERLAPPEDWINDOW,
			 CW_USEDEFAULT, CW_USEDEFAULT,
			 CW_USEDEFAULT, CW_USEDEFAULT,
			 NULL, NULL,
			 hInstance,
			 NULL);

  if (!hWindow)
    return 0;
  
  hListWnd = CreateWindowEx(WS_EX_CLIENTEDGE, "LISTBOX", NULL, 
			    WS_CHILD | WS_VISIBLE | WS_VSCROLL | WS_HSCROLL |
			    WS_BORDER, // | LBS_NOINTEGRALHEIGHT,
			    CW_USEDEFAULT, CW_USEDEFAULT,
			    CW_USEDEFAULT, CW_USEDEFAULT,
			    hWindow, NULL, hInstance, NULL);  
  
  if (!hListWnd) {
    DestroyWindow(hWindow);
    return 0;
  }

  SendMessage(hListWnd,WM_SETFONT,(WPARAM)GetStockObject(DEFAULT_GUI_FONT), 0);
  
  ShowWindow(hWindow, nCmdShow);
  UpdateWindow(hWindow);
  
  return TRUE;
}


int WINAPI WinMain (HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpszCmdParam,
		    int nCmdShow)
{
  MSG msg;
  
  hInstance = hInst;
  if (!InitApplication(hInstance))
    return 0;
  
  if (!InitInstance(hInstance, nCmdShow)) {
    MessageBox(NULL,"InitInstance() failed",NULL,MB_OK);
    return 0;
  }

  SetBreakPoints();
  
  while (GetMessage(&msg, NULL, 0, 0)) {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }
  return msg.wParam;
}
