/*           
Module : PSTAT.CPP
Purpose: Provides a generalised framework for executing a lengthy operation 
         in a thread. Feedback is provided by a progress dialog which can 
         optionally be cancelled.
Author : PJN / PSTAT/1   / 27-03-1997 Initially created
History: NONE

Copyright (c) 1996 PJ Naughter.
All rights reserved.

*/


#ifndef __PSTAT_H__
#define __PSTAT_H__



//////////////////////// Constants ////////////////////////////////////////////

const DWORD PSTAT_CANCEL = 0x0001;  // Display a cancel button on the progress dialog





///// Progress dialog which will be displayed when thread is executing ////////

class CProgressThreadDlg : public CDialog
{
public:
  DECLARE_DYNAMIC(CProgressThreadDlg)

// Constructors
	CProgressThreadDlg(CWnd* pParent = NULL, BOOL bShowCancelButton = FALSE);

// operations
  void SetPercentageDone(int Percentage);
  BOOL HasBeenCancelled() const { return m_bCancelled; };
  void SetCaption(const CString& sCaption);
  void Close();  


protected:
	//{{AFX_DATA(CProgressThreadDlg)
	enum { IDD = IDD_PROGRESS };
	CProgressCtrl	m_ctrlProgress;
	//}}AFX_DATA

  virtual void OnCancel();
  virtual void OnOK();

	//{{AFX_VIRTUAL(CProgressThreadDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CProgressThreadDlg)
	virtual BOOL OnInitDialog();
	afx_msg void OnCancelled();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

  CTime   m_TimeCreation;
  int     m_nCurrentPercentage;
  BOOL    m_bReady;
  CString m_sCaption;
  BOOL    m_bShowCancelButton;
  BOOL    m_bCancelled;
  BOOL    m_bOkToClose;
};


/////////////// format of the user function which can be called ///////////////
typedef ULONG (FUNCTION_WITH_PROGRESS)(void* pData, CProgressThreadDlg* pProgressDlg);


/* Actual function to call to get your function to work in a separate thread 
   with a progress dialog being displayed while it is executing

   Description of parameters taken are:

   pfnFunction:    Your function to execute.
   sProgressTitle: The title that should appear on the progress dialog while your function is executing.
   pData:          Any user defined data that your function will be called with. Similar to the way threads give
                   you a single LPVOID parameter.
   dwFlags:        Bitwise compliment of flags to customize how the function will be executed. Currently only
                   PSTAT_CANCEL is available which will display a cancel button on the progress dialog.
   nPriority:      The thread priority of the thread pfnFunction will execute at. See Win32 help for further
                   details if unclear.
   pParent:        The window which the progress dialog will use as its parent.

   Return value:   TRUE if the function was successfully executed otherwise FALSE i.e. the dialog was cancelled 
                   or some other error occurred.
*/

BOOL ExecuteFunctionWithProgressDialog(FUNCTION_WITH_PROGRESS* pfnFunction, const CString& sProgressTitle, void* pData = NULL, 
                                       DWORD dwFlags = 0, int nPriority = THREAD_PRIORITY_NORMAL, CWnd* pParent = NULL);


#endif //__PSTATDLG_H__
